(* Michael Ramsza (C) 2001            *
 * Warsaw School of Economics         *
 * http://akson.sgh.waw.pl/~mramsz/   *
 * email: mramsz@sgh.waw.pl           *)

BeginPackage["Miscellaneous`Sampling`"]

Game::usage="Game[payoff, strs] is just a form of describing n-player normal form game. Arguments are: \
<payoff> - a list of pure functions being  payoff functions; <strs> - a list of numbers of pure strategies\
for each player. Example: 2-player normal form game would be \
g = Game[{(#[[1]].A.#[[2]]&), (#[[1]].B.#[[2]]&)}, {3,2}] where A and B are matrices 3 by 2 \
containing payoffs for first player - A - and second player - B. First player has three pure strategies, \
second player has two pure strategies. Variable g can be used later on with other functions."

WinningProbability::usage=
  "WinningProbability[game, player, vars] returns a list of winning \
probabilities for player <player> in game <game> in terms of variables \
<vars>. Arguments are: <game> - a game in Game[] form; <player> - number of \
player; <vars> - a list of names of variables."

VarList::usage=
  "VarList[game, vars] returns a list of variables suitable to use with \
Solve[] like modules. Arguments are: <game> - a game in Game[] form; <vars> - \
a list of names ( if the list is {x,y} thr returned list will be sth. like \
{x[1], ..., y[1], ...} depending on number of pure straegies)."

WinningProbabilityDiff::usage=
  "WinningProbabilityDiff[game,vars] gives a list of left hand sides of \
equation discribing 1-sampling equilibrium suitable to construct system of \
differential equation discribing 1-sampling dynamics. Arguments are: <game> - \
describtion of a game in Game[] form; <vars> - a list of names of variables."

SamplingEquilibriumEqs::usage=
  "SamplingEquilibriumEqs[ game, vars, k] returns a system of equations \
describing all k-sampling equlibria. Arguments are: <game> - a game in Game[] \
form; <vars> - a list of names of all variables; <k> - integer discribing \
what k-sampling equilibria we want (Note: this is not implemented yet, hence \
you will get an error message if you specify anything else then 1 here. Since \
1 is a default value you can skip this argument.)."

EquilibriumFilter::usage=
  "EquilibriumFilter[ sol] is a filter removing all complex solutions and \
those not on unit simplex. Suitable to apply on list of solutions returned by \
Solve[]. Arguments are: <sol> - list of solutions returned by Solve[] or \
other modules provided the list is of the same form."

SamplingEquilibriumRoot::usage=
  "SamplingEquilibriumRoot[ game, vars, k] returns the same list as \
SamplingEquilibriumEqs[] except for equations describing unit siplex. \
Argements are the same. Suitable to mix with VarListRoot[] and FindRoot[] \
modules for numerical search of 1-sampling equilibrium."

VarListRoot::usage=
  "VarListRoot[ game, vars, conds] returns a list of numerical initial \
conditions suitable to use as initial condtions for FindRoot[] or NDSolve[]. \
Arguments are: <game> - a game in Game[] form; <vars> - a list of names of \
variables; <conds> - a list of numerical initial conditions describing a \
point on a unit simplex."

Begin["`Private`"]


NextIndex[ index_List, bound_List, pos_Integer]:=
	Module[{lindex, tmpindex},
		lindex = index;
		If[ lindex[[pos]] < bound[[pos]],
			lindex[[pos]]++; 
			Return[lindex],
			If[pos==Length[bound], 
				Return[lindex], 
				tmpindex = NextIndex[lindex,bound,pos+1];
				If[lindex != tmpindex, 
					lindex=tmpindex; 
					lindex[[pos]]=1; 
					Return[lindex],
					Return[lindex] ] ] ]
		];


eVector[index_Integer, dim_Integer]:=
	Module[{lindex,i},
		lindex = Table[0,{i,1,dim}];
		lindex[[index]]=1;
		lindex
		];


Index2Profile[index_List, bound_List]:=
	Module[{lindex,i},
		lindex = Table[eVector[index[[i]], bound[[i]] ], {i,1,Length[bound]} ];
		lindex
		];


NextRealization[ index_List, bound_List, player_Integer, strategy_Integer]:=
	Module[{lindex, lbound},
		lindex = Delete[index,player];
		lbound = Delete[bound, player];
		lindex = NextIndex[ lindex, lbound, 1];
		lindex = Insert[lindex, strategy, player]
		];


StartTree[ bound_List, player_Integer]:=
	Module[{ltree, i,j},
		ltree = Table[1, {i,1,bound[[player]]}, {j,1,Length[bound]}];
		For[i=1, i<=bound[[player]], i++, ltree[[i,player]]=i];
		ltree
		];


ZeroTree[bound_List, player_Integer]:=
	Module[{i,j},
		Table[ 0, {i,1,bound[[player]]}, {j, 1, Length[bound]}]
		];


NextTree[tree_List, bound_List, pos_Integer, player_Integer]:=
	Module[{ltree, lreal, tmpreal, tmptree,i},
		ltree = tree;
		lreal = ltree[[pos]];
		tmpreal = NextRealization[ lreal, bound, player, pos];
		If[ lreal!=tmpreal, ltree[[pos]]=tmpreal; Return[ltree] ];
		If[pos == bound[[player]], Return[ltree] ];
		tmptree = NextTree[ ltree, bound, pos+1, player];
		If[ ltree != tmptree, 
			ltree = tmptree;
			tmpreal = Table[ 1, {i,1 , Length[bound]}]; tmpreal[[player]]=pos;
			ltree[[pos]] = tmpreal;
			Return[ltree],
			Return[ltree]
			]
		];


TreeProbability[ tree_List, player_Integer, vars_List]:=
	Module[{ltree, lvars,i, tmpreal},
		ltree = Transpose[ Delete[ Transpose[tree],player]];
		lvars = Delete[ vars, player];
		For[i=1, i<=Length[ltree],i++,
			tmpreal = Transpose[ {lvars, ltree[[i]] }];
			tmpreal = Flatten[ Map[#[[1]], {#[[2]]}]& /@ tmpreal ]/.List->Times;
			ltree[[i]] = tmpreal;
			];
		ltree/.List->Times
		];


PayoffTree[ tree_List, payoffs_List, bound_List, player_Integer]:=
	Module[{ltree},
		ltree = (Index2Profile[ #, bound]&) /@ tree;
		ltree = payoffs[[player]] /@ ltree
		];




WinningProbability[ gg_Game, player_Integer, vars_List]:=
	Module[{payinfo, i, wp, ptree, nexttree,p},
		wp = Table[ 0, {i, 1, gg[[2, player]]}];
		ptree =ZeroTree[gg[[2]],player];
		nexttree =StartTree[gg[[2]], player];
		While[nexttree!=ptree,
			p = TreeProbability[nexttree, player, vars];
			payinfo = PayoffTree[nexttree, gg[[1]], gg[[2]], player];
			payinfo = Flatten[Position[payinfo, Max[payinfo]]];
			For[i=1, i<=Length[payinfo], i++, 
        wp[[ payinfo[[i]] ]] += p/Length[payinfo]];
			ptree=nexttree;
			nexttree = NextTree[ptree,gg[[2]],1, player];
			];
		wp
		];


VarList[ g_Game, vars_List]:=
	Module[{i, j},
		Flatten[ 
      Table[ vars[[i]][j], {i, 1, Length[ g[[2]] ]}, {j, 1, g[[2, i]] }]]
		];


WinningProbabilityDiff[ gg_Game, vars_List]:=
	Module[{i,j,tmpeqs,eqs},
		eqs={};
			For[i=1, i<=Length[ gg[[2]] ], i++, 
			tmpeqs={};
			tmpeqs = Append[tmpeqs, WinningProbability[gg,i,vars]];
			tmpeqs = Append[tmpeqs, Table[vars[[i]][j], { j,1,gg[[2,i]]}] ];
			tmpeqs = Transpose[tmpeqs];
			tmpeqs = ((Subtract[#[[1]], #[[2]]]) & /@ tmpeqs);
			eqs = Append[eqs,tmpeqs];
			];
		Flatten[eqs]
		];

SamplingEquilibriumEqs::notimplemented = 
  "The support for `1`-sampling equilibrium has not been implemented yet.";


SamplingEquilibriumEqs[ gg_Game, vars_List, k_Integer:1 ]:=
	Module[{i, j, tmpeqs, eqs},
		eqs={};
		If[k!=1, (Message[SamplingEquilibriumEqs::notimplemented,k]); 
      Return[Null]];
		eqs = (Equal[#,0]&) /@ WinningProbabilityDiff[gg,vars];
		For[i=1, i<=Length[ gg[[2]] ], i++,
			tmpeqs = ((Table[ vars[[i]][j], {j,1,gg[[2,i]]}]/.List->Plus)==1);
			eqs = Append[eqs,tmpeqs];
			];
		eqs
		];




FilterComplex[r_List]:=
	Module[{i,rOut},
		rOut=N[r];
		i=1;
		While[i<=Length[rOut],
			If[ MemberQ[ N[ Flatten[ rOut[[i]] /. Rule->List]], _Complex], 
				rOut = Drop[rOut,{i}]; 
				i=1,
				i++,
				i++]
		];
			rOut
		];


FilterNotOnSimplex[r_List]:=
	Module[{ro, i, rotemp},
		ro = N[r];
		NOSQ = ((#<0)||(#>1))&;
		i=1;
		While[i<=Length[ro],
			rotemp =  Drop[ Transpose[ ro[[i]] /. Rule->List], {1}][[1]];
			rotemp = NOSQ /@ rotemp;
			If[MemberQ[rotemp,True], ro = Drop[ro,{i}]; i=1,
				i++, i++];
			]; (* end of while *)
	ro
		];




EquilibriumFilter[ sol_List]:=
	Module[{},
		(Sort /@ Union[FilterNotOnSimplex[  FilterComplex[ sol]]])
		];

SamplingEquilibriumRoot::notimplemented = 
  "The support for `1`-sampling equilibrium has not been implemented yet.";


SamplingEquilibriumRoot[ gg_Game, vars_List,k_Integer:1]:=
	Module[{},
				If[k!=1, (Message[SamplingEquilibriumRoot::notimplemented,k]); 
      Return[Null]];
				(Equal[#,0]&) /@ WinningProbabilityDiff[gg,vars]
		];


VarListRoot[ g_Game, vars_List, conds_List]:=
	Module[{i, j, tmpseq},
		tmpseq={};
		For[i=1, i<=Length[ g[[2]] ], i++,
			For[j=1, j<=g[[2,i]], j++,
				tmpseq = Append[tmpseq, {vars[[i]][j],conds[[i,j]]} ];
				]
			];
		Sequence @@ tmpseq
		];

End[]

EndPackage[]
